#include <QDebug>

#include "Command.hpp"

#define COMMAND_SHELL "C:/Windows/system32/WindowsPowerShell/v1.0/powershell.exe"

Command::Command(QObject *parent)
    : QObject(parent)
    , m_timeout(10)
    , m_shell(COMMAND_SHELL)
    , p_process(nullptr)
{

}

Command::~Command()
{
    clear();
}

QString Command::get() const
{
    return m_command;
}

void Command::set(QString command)
{
    m_command = command;
}

void Command::setTimeout(int seconds)
{
    m_timeout = (seconds < 1 ? 1 : seconds) * 1000;
}

bool Command::syncExecute()
{
    qDebug() << __func__;
    clear();
    m_commandLine.append("-Command");
    m_commandLine.append("Invoke-Command");
    m_commandLine.append("{" + m_command + "}");

    qDebug() << "Running command :" << commandString();

    p_process = new QProcess(this);

    fprintf(stderr, "*** starting process...\n");
    p_process->start(m_shell, m_commandLine);
    fprintf(stderr, "*** wait for finished...\n");

    bool ok = p_process->waitForFinished(m_timeout);
    QProcess::ProcessError processError = p_process->error();

    if (!ok || processError != QProcess::UnknownError) {
        qWarning() << "Error " << processError << "in command :"
                   << commandString();
        return false;
    }

    QProcess::ExitStatus exitStatus = p_process->exitStatus();

    if (exitStatus == QProcess::CrashExit) {
        qWarning() << "Process crashed for command :"
                   << commandString();
        return false;
    }

    m_exitCode = p_process->exitCode();
    m_output = p_process->readAllStandardOutput();
    m_error = p_process->readAllStandardError();
    return true;
}

int Command::exitCode() const
{
    return m_exitCode;
}

QByteArray Command::output() const
{
    return m_output;
}

QByteArray Command::error() const
{
    return m_error;
}

void Command::clear()
{
    //qDebug() << __func__;
    m_exitCode = 0;
    m_output.clear();
    m_error.clear();
    m_commandLine.clear();
    if (p_process) {
        p_process->deleteLater();
    }
}

QString Command::commandString() const
{
    return m_shell + " " + m_commandLine.join(" ");
}
