# SECTION 1
function abort($code, $desc) {
  $json='{"errorCode":'+$code+',"errorDesc":"'+$desc+'","allComputers":[],"allUsers":{}}';
  write-output $json;
  exit;
}

function ldapID($dn) {
  $rep=@();
  $rdn=$dn -split ',';
  foreach($r in $rdn){
    $nv=$r -split '=';
    $rep+=,@($nv[0],$nv[1]);
  }
  $rep;
}

# SECTION 2
$computersdn=(Get-ADComputer -Filter * -SearchBase "OU=Workstation,OU=MyApp,DC=mydomain,DC=local").DistinguishedName;
if(-not $?){
  abort 1 "Error in Get-ADComputer";
}

if($computersdn.Length -eq 0){
  $allComputers='';
}
else{
  $_allComputers=@();
  foreach($cdn in $computersdn){
    $c=ldapID $cdn;
    if($c[0][0] -ne 'CN' -or $c[1][0] -ne 'OU'){
      abort 2 "Invalid LDAP name";
    }

    $computerName=$c[0][1];

    $loggedOnUser=(Get-WmiObject -ComputerName "$computerName" -Class Win32_ComputerSystem).UserName;
    if(-not $?){
      abort 3 "Error in Get-WmiObject";
    }

    $ipAddress=(Get-ADComputer -Filter "Name -eq '$computerName'" -Properties IPv4Address).Ipv4Address;
    if(-not $?) {
      abort 5 "Error in Get-ADComputer";
    }

    if ($loggedOnUser.Length -gt 0 -and $loggedOnUser.LastIndexOf('\') -ge 0){
      $loggedOnUser=$loggedOnUser.Substring($loggedOnUser.LastIndexOf('\') + 1);
    }

    $_allComputers+='{"name":"'+$computerName+'","ipAddress":"'+$ipAddress+'","role":"'+$c[1][1]+'","loggedOnUser":"'+$loggedOnUser+'"}';
  }
  $allComputers=$_allComputers -join ',';
}
#$allComputers='';

# SECTION 3
$users=(Get-ADUser -Filter * -SearchBase "OU=ADU,OU=User,OU=MyApp,DC=mydomain,DC=local");
if(-not $?){
  abort 6 "Error in Get-ADUser";
}

if($users.Length -eq 0){
  $allUsers='';
}
else{
  $_allUsers=@();
  foreach($usr in $users){
    $userName=$usr.UserPrincipalName.Substring(0, $usr.UserPrincipalName.IndexOf('@'));
    $ldapName=$usr.DistinguishedName;

    $u=ldapID $ldapName;
    if($u[0][0] -ne 'CN'){
      abort 7 "Invalid LDAP name";
    }

    $fullName=$u[0][1];

    $groupsdn=(Get-ADPrincipalGroupMembership -Identity "$ldapName").DistinguishedName;
    if(-not $?){
      abort 8 "Error in Get-ADPrincipalGroupMembership";
    }

    if($groupsdn.Length -eq 0){
      $userRoles='';
    }
    else{
      $_userRoles=@();
      foreach($gdn in $groupsdn){
        $g=ldapID $gdn;
        if($g[0][0] -ne 'CN'){
          abort 9 "Invalid LDAP name";
        }
        $_userRoles+=$g[0][1];
      }
      $userRoles='"'+($_userRoles -join '","')+'"';
    }
    $_allUsers+='"'+$userName+'":{"name":"'+$fullName+'","roles":['+$userRoles+']}';
  }
  $allUsers=$_allUsers -join ',';
}
#$allUsers='';

# SECTION 4
$json='{"errorCode":0,"errorDesc":"","allComputers":['+$allComputers+'],"allUsers":{'+$allUsers+'}}';
write-output $json;
exit;
